@@ -1,5 +1,6 @@
 from .helpers import is_async_callable
 from .model import convert_dataclass_to_model, create_parsed_model_field
+from .module_loading import import_string
 from .sequence import find_index, unique
 from .url import join_paths, normalize_path
 
@@ -11,4 +12,5 @@ __all__ = [
     "is_async_callable",
     "normalize_path",
     "unique",
+    "import_string",
 ]
